<?php declare(strict_types=1);

namespace Infomaker\Everyware\Support\Storage;

use Infomaker\Everyware\Support\Collection;

/**
 * Class CollectionDB
 * @package Infomaker\Everyware\Support\Storage
 */
class CollectionDB
{

    /**
     * Contains the option name where the collection is stored
     *
     * @var string
     */
    protected $optionName;

    /**
     * Contains whether the collection is global
     *
     * @var bool
     */
    protected $global = false;

    /**
     * Contains the data collection
     *
     * @var Collection
     */
    private $collection;

    /**
     * CollectionDB constructor.
     *
     *
     * @param string $option_name
     * @param bool   $global
     */
    public function __construct($option_name, $global = false)
    {
        $this->optionName = $option_name;
        $this->global = $global;
        $this->setCollection($this->getCollection());
    }

    /**
     * A way to set the collection
     *
     * @param mixed $collection
     *
     * @return CollectionDB
     */
    public function setCollection($collection = []): CollectionDB
    {
        $this->collection = new Collection($collection);

        return $this;
    }

    /**
     * Retrieve data from ObjectDB depending on global state and convert it to a collection before sending it back
     *
     * @return Collection
     */
    private function getCollection(): Collection
    {
        $collection = $this->global ?
            ObjectDB::getGlobal(null, $this->optionName, []) :
            ObjectDB::get($this->optionName, []);

        return $collection instanceof Collection ? $collection : Collection::make($collection);
    }

    /**
     * Create a collection table with optional data
     *
     * @param string $option_name
     * @param bool   $global
     * @param array  $items
     *
     * @return CollectionDB
     */
    public static function create($option_name, $global = false, array $items = []): CollectionDB
    {
        $collection = new static($option_name, $global);
        $collection->setCollection($items)->save();

        return $collection;
    }

    /**
     * Save collection state to database
     *
     * @return bool
     */
    public function save(): bool
    {
        return $this->global ? ObjectDB::updateGlobal(null, $this->optionName,
            $this->collection) : ObjectDB::update($this->optionName, $this->collection);
    }

    /**
     * Drop the collection table
     *
     * @param string $option_name
     * @param bool   $global
     *
     * @return bool
     */
    public static function drop($option_name, $global = false): bool
    {
        return $global ? ObjectDB::removeGlobal(null, $option_name) : ObjectDB::remove($option_name);
    }

    /**
     * Static function to connect to the options table and retrieve all data as collection
     *
     * @param string $option_name
     *
     * @return Collection
     */
    public static function table($option_name): Collection
    {
        return (new static($option_name))->all();
    }

    /**
     * Retrieve the hole collection of data
     *
     * @return Collection
     */
    public function all(): Collection
    {
        return $this->collection;
    }

    /**
     * Static function to connect to the global options table and retrieve all data as collection
     *
     * @param string $option_name
     *
     * @return Collection
     */
    public static function GlobalTable($option_name): Collection
    {
        return (new static($option_name, true))->all();
    }

    /**
     * Add an item to the collection
     *
     * @param mixed $item
     *
     * @return CollectionDB
     */
    public function add($item): CollectionDB
    {
        $this->collection->push($item);

        return $this;
    }

    /**
     * Merge items into the collection
     *
     * @param $items
     *
     * @return Collection
     */
    public function merge($items): Collection
    {
        $this->setCollection($this->collection->merge($items));

        return $this->all();
    }

    /**
     * Remove an item from the collection
     *
     * @param string $key
     *
     * @return CollectionDB
     */
    public function remove($key): CollectionDB
    {
        $this->collection->forget($key);

        return $this;
    }

    /**
     * Reset collection
     *
     * @return CollectionDB
     */
    public function reset(): CollectionDB
    {
        return $this->setCollection();
    }

    /**
     * Update an existing item in the collection
     *
     * @param $key
     * @param $item
     *
     * @return CollectionDB
     */
    public function update($key, $item): CollectionDB
    {
        $this->collection->put($key, $item);

        return $this;
    }
}
